const express = require('express');
const router = express.Router();
const userService = require('../../db/user-service');
const {authorize} = require('../..//auth/passport-openid-connect/lib/authorize');
const fileService = require('../../db/file-service')


module.exports = function(app, express, passport) {

  /**
   * @desc Gets users
   * @route /api/v1/user
   * @res Users in the database
  */
  router.get('', function(req, res, next) {
    const size = Number(req?.query?.size) || 100
    const index = Number(req?.query?.index) || 0

    userService.getUsers(size, index)
      .then(userList => {
        return res.status(200).json({
          users: userList
        })
      }).catch(err => {
        console.error(err)
        return res.sendStatus(500)
    })
  })
  

  /**
  * @desc Gets the number of users in database
  * @route /api/v1/user/count
  * @res returns 200 with the usercount
  */
  router.get('/count', authorize({ role: ["admin"] }), function(req, res) {
    userService.getUserCount()
      .then(userCount => {
        res.status(200).json({
          count: userCount
      })
      }).catch(err => {
        console.error(err)
        return res.sendStatus(500)
    })
  })

/**
 * @desc Gets the number of favorites for that user
 * @route /api/v1/user/favorite/count
 * @res returns 200 with the favoritecount
*/
  router.get('/favorite/count',authorize({ all: true }), (req, res, next) => {
    if (req?.user?.data?.sub) {
      userService.favorite(req.user.data.sub)
        .then(favorite => {
          res.status(200).json({
            count: favorite.length
          })
        })
    } else {
      res.sendStatus(400)
    }
})

  /**
  * @desc Gets the favorites for that user
  * @route /api/v1/user/favorite
  * @res returns 200 with the favorites
  */
  router.get('/favorite',authorize({ all: true }), (req, res, next) => {
    const feide_id = req?.user?.data?.sub
    const size = Number(req?.query?.size || 100)
    const index = Number(req?.query?.index || 0)
    userService.favorite(feide_id)
      .then(async favorites => {
        let favorite = favorites.slice(size * index, (size + (size * index)))
        let data = await Promise.all(favorite.map(async hash => {               // Go through favorites array with promises
          let filedata = await fileService.findFile(hash)                       // Get the file async 
          if (filedata) {
            return filedata
          } else {
            if (favorites.includes(hash)) {
              favorites.splice(favorites.indexOf(hash), 1)
              userService.changeFavorite(favorites, feide_id)
                .then(() => {})
                .catch(error => {
                  console.log(error)
                  return res.sendStatus(500)
                })
            }
          }
        }))
        let filtered = data.filter((item) => { return item !== undefined })
    
        res.status(200).json({
          files: filtered
        })
      })
  })

  /**
  * @desc sets the favorites for that user
  * @route /api/v1/user/favorite
  * @res returns 200 with the favorites
  */
  
  router.put('/favorite', authorize({ all: true }), (req, res, next) => {
    if (!req?.body?.hash) {
      return res.sendStatus(400)
    }

    userService.favorite(req?.user?.data?.sub)
      .then(favorites => {
        if(favorites) {
          if(favorites.includes(req.body.hash)){
            favorites.splice(favorites.indexOf(req.body.hash), 1)
            userService.changeFavorite(favorites, req.user.data.sub)
              .then(() => {
                res.status(200).json({
                  message: 'Successfully removed favorite'
                })
              })
              .catch(err => {
                console.error(err)
                return res.sendStatus(500)
              })
          } else {
            favorites.push(req.body.hash)
            userService.changeFavorite(favorites, req.user.data.sub)
            .then(() => {
              res.status(200).json({
                message: 'Successfully added favorite'
              })
            })
            .catch(err => {
              console.error(err)
              return res.sendStatus(500)
            })
          }
        } else {
          return res.sendStatus(404)
        }
      })
      .catch(err => {
        console.error(err)
        return res.sendStatus(500)
      })
  })



  /**
   * @desc Authorize a user
   * @route /api/v1/user/profile
   * @res 200 if auth, 500 if error, 401 if unauthenticated
  */
  router.get('/profile', function(req, res, next) {
    if(req?.user) {
      userService.findUser(req.user?.data?.sub)
        .then(user => {
          if (user) {
            req.user['active'] =  user.active                 // Add database properties to user object
            req.user['authored'] = user.authored
            req.user['role'] = user.role
            req.user['tags'] = user.tags
            req.user['favorites'] = user.favorites

            userService.setlastlogin(req.user.data.sub)       // Update latest login
              .then(d => {
                
              })
              .catch(e => {
                console.log(e)
              })
        
              res.status(200).json({
                feide_id: req.user.data.sub,
                name: req.user.data.name,
                photo: req.user.data.picture,
                token: req.user.token.access_token,
                groups: req.user.groups,
                active: req.user.active,
                authored: req.user.authored,
                role: req.user.role,
                tags: req.user.tags,
                favorites: req.user.favorites
              })
          } else {
            res.status(500).json({
              message: 'An error occured'
            })
          }
        })
        .catch(error => {
          console.log(error)
        })

    } else {
      res.sendStatus(401)
    }
  })

  
  //ALL WILDCARDS BELOW THIS LINE!!!!!!!!!!!
  /**
   * @desc Edits a user
   * @route /api/v1/user/:id of user
   * @res 200 if user updated, 404 if user not found
   */
  router.put('/:id', authorize({ role: ["admin"] }),(req, res, next) => {
    if (req?.params?.id && req?.body) {
      userService.editUser(req.body, req.params.id)
        .then(user => {
          if (user) {
            res.sendStatus(204)
          } else {
            res.sendStatus(404)
          }
        })
        .catch(error => {
          console.log(error)
          res.sendStatus(500)
        })
    } else {
      res.sendStatus(400)
    }
  })

  /**
   * @desc Deletes a user
   * @route /api/v1/user/:id
   * @res 200 if user deleted, 404 if not found, 403 if not allowed
  */
  router.delete('/:id', authorize({ role: ["admin"] }),function(req, res, next) {
    if(req?.params?.id){
      userService.findUser(req.params.id)
        .then(user => {
          if(user) {
            userService.deleteUser(req.params.id)
              .then(() => {
                return res.sendStatus(204)
              })
              .catch(err => {
                console.error(err)
                return res.sendStatus(500)
              })
          } else {
            return res.sendStatus(404)
          }
        })
        .catch(err => {
        console.error(err)
        return res.sendStatus(500)
      })
    } else {
      return res.sendStatus(400)
    }
  })

  /**
   * @desc Gets a specific user
   * @route /api/v1/:id
   * @res Users in the database
  */
  router.get('/:id', authorize({ role: ["admin"] }), function(req, res, next) {
  if (req?.params?.id) {
    userService.findUser(req.params.id)
      .then(user => {
        if (user) {
          return res.status(200).json({
            id: user._id,
            username: user.username,
            role: user.role
          })
        } else {
          return res.sendStatus(404)
        }
      }).catch(err => {
        console.error(err)
        return res.sendStatus(500)
      })
    } else {
      return res.sendStatus(400)
    }
  })


  return router
}


