const File = require('../models/file')
const virustotal = require('../pipeline/virustotal/virustotal')
const { get_malwarebazaar } = require('../pipeline/malwarebazaar/malwarebazaar')
const { spawn } = require('child_process')
const hasha = require('hasha')
const path = require('path')
const util = require('util')
const exec = util.promisify(require('child_process').exec)
const sysinfo = require('../pipeline/sysinfo')
const fileService = require('./file-service')
const logger = require('../logging/infologger')
const fsPromises = require('fs').promises
const CONTAINERCOUNT = Number(process.env.containercount)

class pipelineService {
  constructor() {
    this.limit = 10000
    this.queue = []
    //set ratelimiter to fire every minute
    this.timerObj = setInterval(this.dockerWorker.bind(this), this.limit)
  }

  async addAllSources(fileclass) {
    console.log('adding all sources')
    let files = await fileclass.listFiles()

    files.forEach(async (element) => {
      let file_sha256 = await hasha.fromFile(
        fileclass.currentPath + '/' + element,
        { algorithm: 'sha256' }
      )
      virustotal.addToQueue(file_sha256)
      //this.malwarebazaar_addSource(file_sha256)
    })

    this.dockerAddToQueue(fileclass)
  }

  /**
   * @desc Adds malwarebazaar data to documents
   * @param string $hash - The HASH of the file where data will be added
   */

  async malwarebazaar_addSource(hash) {
    var data = await get_malwarebazaar(hash)
    if (data) {
      File.findOneAndUpdate(
        { sha256: data.resource },
        { 'analyzed_info.source': 'malwarebazaar', 'analyzed_info.data': data }
      )
        .then(() => {
          console.log('added malwarebazaar')
        })
        .catch(() => {})
    } else {
      console.log('No malwarebazaar data to add')
    }
  }

  async docker_unzip(fileclass) {
    return new Promise(async (resolve, reject) => {
      //getting the curent user. Docker needs the correct file permissions to work
      let currentuser = Number((await exec('id -u')).stdout)
      let currentgroup = Number((await exec('id -g')).stdout)

      console.log('path to upload: ', path.resolve('upload/'))
      const docker = spawn('docker', [
        'run',
        '--name', fileclass.id,
        '--user', `${currentuser}:${currentgroup}`,
        '--env', `FILEFOLDER=${fileclass.id}`,
        '-v', `${path.resolve('upload/')}:/upload`,
        'unzip:latest',
        '/bin/sh', 'entrypoint.sh',
      ])

      docker.stdout.on('data', (data) => {
        console.log(`stdout:\n${data}`)
      })

      docker.stderr.on('data', (data) => {
        console.error(`stderr: ${data}`)
      })

      docker.on('error', (error) => {
        console.error(`error: ${error.message}`)
      })

      docker.on('close', (data) => {
        exec(`docker rm ${fileclass.id}`)
        resolve()
      })
      //reject(error)
    }).catch((err) => {
      console.log(err)
    })
  }

  async docker_analyze(fileclass) {
    return new Promise(async (resolve, reject) => {
      //getting the curent user. Docker needs correct file permissions to work
      let currentuser = process.getuid()
      let currentgroup = process.getgid()

      console.log('analyzing: ', fileclass.id)
        const docker = spawn('docker', [
          'run',
          '--name', fileclass.id,
          '--user', `${currentuser}:${currentgroup}`,
          '--env', `FILEFOLDER=${fileclass.id}`,
          '-v', `${path.resolve('upload/')}:/upload`,
          'analyze:latest',
          '/bin/sh', 'entrypoint.sh',
        ])

      docker.stdout.on('data', (data) => {
        //console.log(`stdout:\n${data}`)
      })

      docker.stderr.on('data', (data) => {
        console.error(`stderr: ${data}`)
      })

      docker.on('error', (error) => {
        console.error(`error: ${error.message}`)
        reject(error)
      })

      docker.on('close', async (code) => {
        fileclass.updateUploadStatus('analyzed', true)
        fileclass.addToStorage()
        await this.addAnalyzeSource(fileclass.id)
        console.log(`child process exited with code ${code}`)
        exec(`docker rm ${fileclass.id}`).catch(() => {})

        resolve()
      })

      //reject(error)
    }).catch((err) => {
      console.log(err)
    })
  }

  async dockerAddToQueue(fileclass) {
    if (!this.queue.includes(fileclass)) {
      let files = await fileclass.listFiles()

      files.forEach(async file => {
        let currentFile = fileclass.currentPath + '/' + file
        let file_sha256 = await hasha.fromFile(currentFile, {
          algorithm: 'sha256',
        })
        fileService.setPendingAnalysis(file_sha256, 'static_analysis')
      })
      this.queue.push(fileclass)
    }
  }

  async dockerWorker() {
    let load = await sysinfo.getLoad()
    let memoryLoad = load.memoryLoad
    let cpuLoad = load.cpuLoad

    logger.info({
      'message': 'dockerQueue',
      'system': 'docker',
      'length': this.queue.length,
      'queue': this.queue.map(fileclass => fileclass.id),
      'cpuLoad': cpuLoad,
      'memoryLoad': memoryLoad
    })

    this.queue.map((fileclass, index) => {
      fileclass.updateUploadStatus('analyze_queue', index)
    })

    //cpuLoad < 0.95 && 
    if (cpuLoad < 0.60 && memoryLoad < 0.75) {
      let smallest =  CONTAINERCOUNT < this.queue.length ? CONTAINERCOUNT : this.queue.length
      for (let i = 0; i < smallest; i++) {

        let fileclass = this.queue.shift()
        if (fileclass.docker == 'unzip') {
          this.docker_unzip(fileclass)
        } else if (fileclass.docker == 'analyze') {
          //setting timeout to spread spawn of docker containers
          setTimeout(() => {
            this.docker_analyze(fileclass)
          }, (this.limit/smallest)*i);
        }
      }
    }
  }

  async addAnalyzeSource(id){
    let folders = await fsPromises.readdir(`./upload/analysis_info/${id}`).catch((err) => {
      console.log(`cant read folder: ./upload/analysis_info/${id}`, err)
      return
    })
    await Promise.all(
      folders.map(async (folder) => {
      let files = await fsPromises.readdir(`./upload/analysis_info/${id}/${folder}`).catch((err) => {
        console.log(`cant read folder: ./upload/analysis_info/${id}/${folder}`, err)
      })
      await Promise.all(
        files.map(async (file) => {
          fileService.unsetPendingAnalysis(file, 'static_analysis')
          let data = await fsPromises.readFile(`./upload/analysis_info/${id}/${folder}/${file}`, 'utf-8').catch((err) => {
            console.log(`cant read ./upload/analysis_info/${id}/${folder}/${file}`, err)
          })
          await fileService.addAnalysisToDb(file, folder, JSON.parse(data))
          return 
        }))
      }))
    await fsPromises.rm(`./upload/analysis_info/${id}`, { recursive: true, force: true }).catch((err) => {
      console.log('cant remove folder. ID: ', id, err)
    })
    return
  }
}

module.exports = new pipelineService()
