const {get} = require('axios')
const logger = require('../../logging/infologger')
const fileService = require('../../db/file-service') 

/**
 * @desc Get information about a certain file from virustotal
 * @param string $hash - The HASH of the file 
 * @return Promise - Virustotal data of the file
*/  

class virustotal {

  constructor() {
    logger.info('started virustotal')
    this.limit = 60000
    this.queue = []

    this.dayUsed = 0
    //set ratelimiter to fire every minute
    this.timerObj = setInterval(this.rateLimiter.bind(this), this.limit)
  }

  async getData(hash) {
    let apikey = process.env.virustotal_apikey
    let response = ''
    if (!apikey){
      console.log('no apikey')
      //WINSTON ERROR
    }
    else{
      console.log('fetching virustotal for: ', hash)
      try {
        response = await get(`https://www.virustotal.com/api/v3/files/${hash}`, { 
          headers: {
            'x-apikey': apikey
          },
          validateStatus: false
        })
      }catch (error){
        console.error(error)
      }

      if (response.status == 200) {
        //got analysis, add to db
        fileService.unsetPendingAnalysis(hash, 'virustotal')
        this.addSource(hash, response.data.data)
      } else if (response.status == 429) {
        //daily/monthly quota reached. add hash back to queue
        this.addToQueue(hash)
      } else if (response.status == 404) {
        //file not found on virustotal
        fileService.unsetPendingAnalysis(hash, 'virustotal')
        this.addSource(hash, null)
        //console.log(response)
      } else {
        logger.info({
          'message': 'virustotalBadResponse',
          'system': 'virustotal',
          'statusCode': response.status
        })
      }
    }
  }

  addToQueue(hashes) {
    let array = (Array.isArray(hashes)) ? hashes : [hashes]
    array.map(hash => {
      if (!this.queue.includes()) {
        fileService.setPendingAnalysis(hash, 'virustotal')
        this.queue.push(hash)
      }
    })
  }

  async addSource(hash, data) {
    fileService.addAnalysisToDb(hash, 'virustotal', data)
      .then((ret) => {
          console.log('added virustotal')
      }).catch((err) => {
          console.log(err)
      })
}   

  rateLimiter () {
    if (this.dayUsed < process.env.virustotal_daylimit){
        let smallest =  process.env.virustotal_minutelimit < this.queue.length ? process.env.virustotal_minutelimit : this.queue.length
        logger.info({
          'message': 'virustotalQueue',
          'system': 'virustotal',
          'length': this.queue.length,
          'queue:': this.queue,
          'dailyQuotaUse': this.dayUsed
        })

        for (let i = 0; i < smallest; i++) {
          this.getData(this.queue.shift())
          this.dayUsed += 1
        }
    

    }else{
      console.log("daily virustotal limit exceeded. Setting timeout to next day")
      
      // stop ratelimiter from firing every minute when daily limit is exceeded
      clearInterval(this.timerObj)

      var midnight = new Date()
      midnight.setHours(24,0,0,0)
      var msToMidnight = midnight - Date.now()

      //set timeout to next day
      setTimeout(() => {
        this.timerObj = setInterval(this.rateLimiter.bind(this), this.limit)
      }, msToMidnight)
    }

  }

}

module.exports = new virustotal()
